#! /usr/bin/perl

use warnings;
use strict;
use CGI::Carp 'fatalsToBrowser';


# Use the CGI library
use CGI;
$CGI::DISABLE_UPLOADS = 1; # No file uploads
$CGI::POST_MAX = 512 * 1024; # Form may not exceed 512KiB
my $cgi = new CGI;

# Use our RC4 library
use CSCI162_RC4;

# HTTP header
print $cgi->header(-charset => 'UTF-8');

# This string represents a float: left break
my $break = q(<div class="breaker"><hr /></div>);

# Begin the HTML.
print qq(<?xml version="1.0" encoding="utf-8" ?> 
<!DOCTYPE html PUBLIC "-//W3C//DTD XHTML 1.1//EN" "http://www.w3.org/TR/xhtml11/DTD/xhtml11.dtd">
<html>
<head>
<title>ARCFOUR Web Interface</title>
<link href="stream.css" rel="stylesheet" type="text/css" />
</head>
<body>
<div>
	<div id="image"><img src="encryption.gif" alt="Key and lock" /></div>
	<div id="title">
	<h1>ARCFOUR Web Interface</h1>
	<h2>CSCI 162 Cryptography Spring 2005</h2>
	<h2>Dr. Poorvi Vora</h2>
	<h2>Anasse Bari and Peter S. May</h2>
	</div>
</div>
$break
);

# An error string we may need
my $error = '';

# Did we receive anything to translate?
if( defined $cgi->param('message') ) {
	# Yes.
	# Do something.
	TRANSLATE: {
		my $key = $cgi->param('key');
		length($key) > 0
			or ($error="Key must be at least one character.", last TRANSLATE);
		my $message = $cgi->param('message');
		my $encoding = lc $cgi->param('encoding');
		# But do we recognize it?
		$encoding =~ /^(?:hexadecimal|base64)$/o
			or ($error="Unrecognized encoding.", last TRANSLATE);
			
		my $mode;
		{
			# Encrypt or decrypt?
			if( $cgi->param('encryptSubmit') ) {
				$mode = 'e';
			} elsif( $cgi->param('decryptSubmit') ) {
				$mode = 'd';
			} else {
				# Neither?  Skip the translate.
				$error = "Direction not specified.";
				last TRANSLATE;
			}
		}

		# Throw out the garbage (characters).
		if( $mode eq 'd' ) {
			if( $encoding eq 'hexadecimal' ) {
				$error .= " Non-hexadecimal characters in input were removed." if $message =~ s/[^0-9A-Fa-f]+//osg;
			} elsif( $encoding eq 'base64' ) {
				$error .= " Non-Base64 characters in input were removed." if $message =~ s![^A-Za-z0-9+/=]+!!osg;
			}
		}

		# Which sub to use?
		my $sub = {
			'base64.e' => \&rc4_encrypt_base64,
			'base64.d' => \&rc4_decrypt_base64,
			'hexadecimal.e' => \&rc4_encrypt,
			'hexadecimal.d' => \&rc4_decrypt
		}->{"$encoding.$mode"};
		unless(defined $sub) {
			$error ="Invalid encoding/direction combination.";
			last TRANSLATE;
		}

		# Do it.
		my $result = $sub->($key,$message);

		# Make sure the result contains no bogus characters.
		$error .= " Non-ASCII characters in output were replaced." if $result =~ s/[^\x20-\x7f\x0a\x0d]/?/osg;

		# Put the result back into the form.
		$cgi->param('message',$result);
	}
}


{
	# Print an explanation.
	print q#
<div>
<p>This program accepts a key and a stream of text and operates on it using the ARCFOUR
(unofficial implementation of RC4) stream cipher
as implemented in our project.  The encrypted form of a message generally contains some non-text
characters that possibly won't be handled correctly by the web browser.  Therefore, the encrypted
form can be represented in either hexadecimal digits or in Base64 encoding.</p>
<p>This application is a demonstration only; using it on a remote server will allow your message
to travel over public channels in the clear!</p>
<ul>
	<li><a href="CSCI162_RC4.pm.txt">View source of ARCFOUR module</a></li>
	<li><a href="stream.txt">View source of interface</a></li>
	<li><a href="CSCI162_RC4.html">View POD documentation of module</a></li>
</ul>
</div>
#;
}

# Print an error
if($error ne '') {
	print "<div class='error'>$error</div>\n";
}

print "<div>";

# Provide a form
print $cgi->start_form(-method=>'post',-action=>'stream.cgi');

print "<div><div class='leftPart'><div class='leftPartContent'>Key</div></div><div class='rightPart'>";
print $cgi->textfield(-name=>'key',-value=>"This key may be any random string of characters.",
	-maxLength=>256, -class=>"rightPartInput");
print "</div>$break</div>\n";

print "<div><div class='leftPart'><div class='leftPartContent'>Message</div></div><div class='rightPart'>";
print $cgi->textarea(-name=>'message',-value=>"Enter the message (plaintext or ciphertext) here.",
	-class=>'rightPartInput',-rows=>10,-cols=>80);
print "</div>$break</div>\n";

{
	my @radios = $cgi->radio_group(-name => 'encoding',
		-values => [qw/Hexadecimal Base64/],
		-default => 'Hexadecimal'
	);
	$_ = "<div>$_</div>" foreach @radios;
	print "<div><div class='leftPart'><div class='leftPartContent'>Encoding</div></div><div class='rightPart'>@radios</div>$break</div>\n";
}

print "<div class='submissions'>";
print $cgi->submit(-name=>'encryptSubmit',-value=>"Encrypt",-class=>'submitButton');
print $cgi->submit(-name=>'decryptSubmit',-value=>"Decrypt",-class=>'submitButton');
print "</div>";

$_ = $cgi->end_form;
s!(<input\s.*?>)!<div class="hide">$1</div>!sg;
print $_;

print "</div>";
print q(<div class='validations'><a href="http://validator.w3.org/check?uri=referer"><img src="http://www.w3.org/Icons/valid-xhtml11" alt="Valid XHTML 1.1" height="31" width="88" /></a>
<a href="http://jigsaw.w3.org/css-validator/check/referer"><img src="http://jigsaw.w3.org/css-validator/images/vcss" alt="Valid CSS" /></a>

</div>);
print "</body></html>";


